#------------------------------------------------------------------------------
# <auto-generated>
#     This file was auto generated.
#
#     Changes to this file may cause incorrect behavior and will be lost if
#     the code is regenerated.
# </auto-generated>
#------------------------------------------------------------------------------
import typing
from abc import abstractmethod, ABCMeta
from Oasys.Collections import IList
from OasysUnits import Pressure, Strain


# noinspection PyPep8Naming
class ITensionCompressionCurve(metaclass=ABCMeta):
    """A stress-strain relationship for a "IMaterial".

    See Glossary
    """

    @property
    def Compression(self) -> IStressStrainCurve:
        """The compression stress-strain curve.

        :exception System.ArgumentNullException: If Compression is set to null
        """
        pass

    @property
    def Tension(self) -> IStressStrainCurve:
        """The tension stress-strain curve.

        :exception System.ArgumentNullException: If Tension is set to null
        """
        pass

    @staticmethod
    def Create(tension: IStressStrainCurve, compression: IStressStrainCurve) -> ITensionCompressionCurve:
        """Creates a new "ITensionCompressionCurve" from
         "Property Tension" and "Property Compression" stress-strain curves.

        :param tension: The tension stress-strain curve.
        :type tension: IStressStrainCurve
        :param compression: The compression stress-strain curve.
        :type compression: IStressStrainCurve
        :return: Stress-strain relationship with the specified tension and compression curves.
        :rtype: ITensionCompressionCurve

        :exception System.ArgumentNullException: If tension is null
        :exception System.ArgumentNullException: If compression is null
        """
        pass

    @abstractmethod
    def StressAt(self, strain: Strain) -> Pressure:
        """The stress at a given strain.

        :param strain: The strain at which stress is to be calculated.
        :type strain: Strain
        :return: Compressive stress for a negative strain
        and tensile stress for a positive strain.
        :rtype: Pressure
        """
        pass


# noinspection PyPep8Naming
class IStressStrainCurve(metaclass=ABCMeta):
    """A stress-strain curve.
    The stress-strain inputs must have positive and finite values.
    It can be used to define the tension or the compression
    part of an "ITensionCompressionCurve".

    See Glossary
     "IStressStrainCurve" is a base class for other stress-strain curve types,
    e.g. "ILinearStressStrainCurve" and "IRectangularStressStrainCurve".
    The nature of the stress-strain curve is considered to be
    tensile or compressive based on the parameter of
     "ITensionCompressionCurve.Create(IStressStrainCurve, IStressStrainCurve)"
    to which it has been assigned.
    """

    @property
    def FailureStrain(self) -> Strain:
        """The strain at which the material fails.
        """
        pass

    @abstractmethod
    def StressAt(self, strain: Strain) -> Pressure:
        """The stress at a given strain.

        :param strain: The strain at which stress is to be calculated.
        :type strain: Strain
        :return: Stress at given strain.
        :rtype: Pressure
        """
        pass


# noinspection PyPep8Naming
class IBilinearStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" in which the stress is
    directly proportional to the strain until the yield point. The
    curve has a constant gradient between the yield and the failure points.
    """

    @property
    def FailurePoint(self) -> IStressStrainPoint:
        """The point at which the material fails.

        :exception System.ArgumentNullException: If FailurePoint is set to null
        """
        pass

    @property
    def YieldPoint(self) -> IStressStrainPoint:
        """The point up to which the stress-strain relationship is elastic.

        :exception System.ArgumentNullException: If YieldPoint is set to null
        """
        pass

    @staticmethod
    @typing.overload
    def Create(yieldPoint: IStressStrainPoint, failurePoint: IStressStrainPoint) -> IBilinearStressStrainCurve:
        """Creates a new "IBilinearStressStrainCurve"
        with a constant gradient between the yield and the failure points.

        :param yieldPoint: The point up to which the stress-strain relationship is elastic, and
        beyond which the curve has a constant gradient.
        :type yieldPoint: IStressStrainPoint
        :param failurePoint: The point at which the material fails.
        :type failurePoint: IStressStrainPoint

        :exception System.ArgumentException: When the yield strain is larger than the failure strain.
        :exception System.ArgumentException: When the yield stress is larger than the failure stress.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If yieldPoint is null
        :exception System.ArgumentNullException: If failurePoint is null
        """
        pass

    @staticmethod
    @typing.overload
    def Create(yieldPoint: IStressStrainPoint, failureStrain: Strain) -> IBilinearStressStrainCurve:
        """Creates a new "IBilinearStressStrainCurve"
        with a zero gradient between the yield and the failure points.

        :param yieldPoint: The point up to which the stress-strain relationship is elastic, and
        beyond which there is no change in stress with change in strain.
        :type yieldPoint: IStressStrainPoint
        :param failureStrain: The strain at which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the yield strain is larger than the failure strain.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If yieldPoint is null
        """
        pass


# noinspection PyPep8Naming
class IStressStrainPoint(metaclass=ABCMeta):
    """A stress-strain point.
    It can be used as an input type to define a "IStressStrainCurve".
    """

    @property
    def Strain(self) -> Strain: ...

    @property
    def Stress(self) -> Pressure: ...

    @staticmethod
    def Create(stress: Pressure, strain: Strain) -> IStressStrainPoint:
        """Creates a new "IStressStrainPoint".
        """
        pass


# noinspection PyPep8Naming
class IExplicitStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" defined by a list of
     "IStressStrainPoint"s.

    See Glossary
    """

    @property
    def Points(self) -> IList[IStressStrainPoint]:
        """The stress-strain points.

        :exception System.ArgumentException: When the first point in the stress-strain points list is not a zero stress and zero strain point.
        :exception System.ArgumentException: When the points in the stress-strain points list are not in the ascending order of the strains.
        :exception System.ArgumentException: When the stress-strain points list has fewer than two points.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If Points is set to null
        """
        pass

    @Points.setter
    def Points(self, value: IList[IStressStrainPoint]) -> None: ...

    @staticmethod
    def Create() -> IExplicitStressStrainCurve:
        """Creates a new "IExplicitStressStrainCurve".

        :return: Explicit curve with empty list of stress-strain points ( "Property Points").
        :rtype: IExplicitStressStrainCurve
        """
        pass


# noinspection PyPep8Naming
class IFibModelCodeStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """Schematic "IStressStrainCurve" from FIB model code.
    Refer AdSec Theory Manual
    for further details.
    FIB schematic curve is commonly used for concrete materials.
    """

    @property
    def PeakPoint(self) -> IStressStrainPoint:
        """The point of maximum stress.

        :exception System.ArgumentNullException: If PeakPoint is set to null
        """
        pass

    @property
    def InitialModulus(self) -> Pressure:
        """The initial gradient of the stress-strain curve.
        """
        pass

    @staticmethod
    def Create(initialModulus: Pressure, peakPoint: IStressStrainPoint, failureStrain: Strain) -> IFibModelCodeStressStrainCurve:
        """Creates a new "IFibModelCodeStressStrainCurve" stress-strain curve
        (schematic stress-strain curve from FIB model code).

        :param initialModulus: The initial gradient of the stress-strain curve.
        :type initialModulus: Pressure
        :param peakPoint: The point of maximum stress.
        :type peakPoint: IStressStrainPoint
        :param failureStrain: The strain beyond which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the peak strain is larger than the failure strain.
        :exception System.ArgumentException: When the initial gradient is not a poitive finite value.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If peakPoint is null
        """
        pass


# noinspection PyPep8Naming
class ILinearStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" in which the stress
    is directly proportional to the strain. The material provides no strength
    beyond the failure point.
    """

    @property
    def FailurePoint(self) -> IStressStrainPoint:
        """The point at which the material fails.

        :exception System.ArgumentNullException: If FailurePoint is set to null
        """
        pass

    @staticmethod
    def Create(failurePoint: IStressStrainPoint) -> ILinearStressStrainCurve:
        """Creates a new "ILinearStressStrainCurve".

        :param failurePoint: The point at which the material fails.
        :type failurePoint: IStressStrainPoint

        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If failurePoint is null
        """
        pass


# noinspection PyPep8Naming
class IManderConfinedStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" for a confined concrete "IMaterial".
    Refer 'Mander J.B., Priestley M.J.N., and Park R. (1988). "Theoretical
    Stress-Strain Model for Confined Concrete", ASCE Journal of Structural
    Engineering, V. 114, No. 8, pp. 1804-1826' for further details.
    Mander curve is commonly used for concrete materials.
    """

    @property
    def InitialModulus(self) -> Pressure:
        """The initial gradient of the stress-strain curve.
        """
        pass

    @property
    def ConfinedStrength(self) -> Pressure:
        """The confined strength of the material.
        """
        pass

    @property
    def UnconfinedStrength(self) -> Pressure:
        """The unconfined strength of the material.
        """
        pass

    @staticmethod
    def Create(unconfinedStrength: Pressure, confinedStrength: Pressure, initialModulus: Pressure, failureStrain: Strain) -> IManderConfinedStressStrainCurve:
        """Creates a new "IManderConfinedStressStrainCurve".
        Peak strain is calculated from unconfined strength, confined strength and initial modulus.

        :param unconfinedStrength: The unconfined strength of the material.
        :type unconfinedStrength: Pressure
        :param confinedStrength: The confined strength of the material.
        :type confinedStrength: Pressure
        :param initialModulus: The initial gradient of the stress-strain curve.
        :type initialModulus: Pressure
        :param failureStrain: The strain beyond which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the peak strain is larger than the failure strain.
        :exception System.ArgumentException: When the initial gradient is not a poitive finite value.
        :exception System.ArgumentException: When the confined strength is smaller than the unconfined strength.
        :exception System.ArgumentException: When the failure strain is smaller than the calculated peak strain.
        """
        pass


# noinspection PyPep8Naming
class IManderStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" for an unconfined concrete "IMaterial".
    Refer 'Mander J.B., Priestley M.J.N., and Park R. (1988). "Theoretical
    Stress-Strain Model for Confined Concrete", ASCE Journal of Structural
    Engineering, V. 114, No. 8, pp. 1804-1826' for further details.
    Mander curve is commonly used for concrete materials.
    If the failure strain is less than two times the peak strain,
    the curve doesn't reach the zero stress point and it ends at
    the failure strain.
    """

    @property
    def PeakPoint(self) -> IStressStrainPoint:
        """The point of maximum stress.

        :exception System.ArgumentNullException: If PeakPoint is set to null
        """
        pass

    @property
    def InitialModulus(self) -> Pressure:
        """The initial gradient of the stress-strain curve.
        """
        pass

    @staticmethod
    def Create(initialModulus: Pressure, peakPoint: IStressStrainPoint, failureStrain: Strain) -> IManderStressStrainCurve:
        """Creates a new "IManderStressStrainCurve" stress-strain curve.

        :param initialModulus: The initial gradient of the stress-strain curve.
        :type initialModulus: Pressure
        :param peakPoint: The point of maximum stress.
        :type peakPoint: IStressStrainPoint
        :param failureStrain: The strain beyond which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the failure strain is smaller than twice the peak strain.
        :exception System.ArgumentException: When the initial gradient is not a positive and finite value.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If peakPoint is null
        """
        pass


# noinspection PyPep8Naming
class IParabolaRectangleStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" which is initially parabolic.
    The stress is constant beyond the yield point. The material provides no strength beyond
    the failure point.
    Parabola-rectangle curve is commonly used for concrete materials.
    """

    @property
    def YieldPoint(self) -> IStressStrainPoint:
        """The point up to which the stress-strain relationship is parabolic,
        and beyond which there is no change in stress with change in strain.

        :exception System.ArgumentNullException: If YieldPoint is set to null
        """
        pass

    @staticmethod
    def Create(yieldPoint: IStressStrainPoint, failureStrain: Strain) -> IParabolaRectangleStressStrainCurve:
        """Creates a new "IParabolaRectangleStressStrainCurve".
        The initial gradient of the curve is set such that the parabola has
        zero gradient at the yield point.

        :param yieldPoint: The point up to which the stress-strain relationship is parabolic, and beyond which
        there is no change in stress with change in strain.
        :type yieldPoint: IStressStrainPoint
        :param failureStrain: The strain beyond which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the yield strain is larger than the failure strain.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If yieldPoint is null
        """
        pass


# noinspection PyPep8Naming
class IParkStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" with a linear stress-strain
    relationship up to the yield point. The gradient of the curve is zero
    up to some point beyond the yield, and then the curve rises to reach the peak
    point. The point up to which the gradient of the curve is zero is calculated
    as:

    stress = yield stress,strain = yield strain * factor,factor = 1, for yield stress > 120ksi,= [-4 * (yield stress in ksi - 60) / 60 + 5], for yield stress between 60ksi and 120ksi,= [-9 * (yield stress in ksi - 40) / 20 + 14], for yield stress between 40ksi and 60ksi,= 14, for yield stress less than 40ksi.

    Refer AdSec Theory Manual
    for further details.
    """

    @property
    def YieldPoint(self) -> IStressStrainPoint:
        """The point up to which the stress-strain relationship is elastic.

        :exception System.ArgumentNullException: If YieldPoint is set to null
        """
        pass

    @staticmethod
    def Create(yieldPoint: IStressStrainPoint) -> IParkStressStrainCurve:
        """Creates a new "IParkStressStrainCurve".

        :param yieldPoint: The point up to which the stress-strain relationship is elastic.
        :type yieldPoint: IStressStrainPoint

        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If yieldPoint is null
        """
        pass


# noinspection PyPep8Naming
class IPopovicsStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" based on the work of Popovics.
    It is defined by a series of curves adjusted on the basis of Thorenfeldt base curve.
    Refer AdSec Theory Manual
    for further details.
    Popovics curve is commonly used for concrete materials.
    """

    @property
    def PeakPoint(self) -> IStressStrainPoint:
        """The point of maximum stress.

        :exception System.ArgumentNullException: If PeakPoint is set to null
        """
        pass

    @staticmethod
    def Create(peakPoint: IStressStrainPoint, failureStrain: Strain) -> IPopovicsStressStrainCurve:
        """Creates a new "IPopovicsStressStrainCurve".

        :param peakPoint: The point of maximum stress.
        :type peakPoint: IStressStrainPoint
        :param failureStrain: The strain beyond which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the peak strain is larger than the failure strain.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If peakPoint is null
        """
        pass


# noinspection PyPep8Naming
class IRectangularStressStrainCurve(IStressStrainCurve, metaclass=ABCMeta):
    """A type of "IStressStrainCurve" in which the stress
    is zero until the yield point. The stress is constant betweeen the yield
    point and the failure point.
    Rectanglular curve is commonly used for concrete materials.
    """

    @property
    def YieldPoint(self) -> IStressStrainPoint:
        """The material does not have strength until the
        yield point. The stress is constant beyond this point.

        :exception System.ArgumentNullException: If YieldPoint is set to null
        """
        pass

    @staticmethod
    def Create(yieldPoint: IStressStrainPoint, failureStrain: Strain) -> IRectangularStressStrainCurve:
        """Creates a new "IRectangularStressStrainCurve".

        :param yieldPoint: The material does not have strength until the yield point.
        The stress is constant beyond this point.
        :type yieldPoint: IStressStrainPoint
        :param failureStrain: The strain beyond which the material fails.
        :type failureStrain: Strain

        :exception System.ArgumentException: When the yield strain is larger than the failure strain.
        :exception System.ArgumentException: When any of the stress-strain inputs is negative or infinite.
        :exception System.ArgumentNullException: If yieldPoint is null
        """
        pass
