#------------------------------------------------------------------------------
# <auto-generated>
#     This file was auto generated.
#
#     Changes to this file may cause incorrect behavior and will be lost if
#     the code is regenerated.
# </auto-generated>
#------------------------------------------------------------------------------
import typing
from abc import abstractmethod, ABCMeta
from Oasys.AdSec.Materials.StressStrainCurves import ITensionCompressionCurve
from OasysUnits import Pressure


# noinspection PyPep8Naming
class IMaterial(metaclass=ABCMeta):
    """A base class for all material types.

    See Glossary
    """

    @property
    def Serviceability(self) -> ITensionCompressionCurve:
        """The material's stress-strain relationship for serviceability (SLS) analysis.

        :exception System.ArgumentNullException: If Serviceability is set to null
        """
        pass

    @property
    def Strength(self) -> ITensionCompressionCurve:
        """The material's stress-strain relationship for strength (ULS) analysis.

        :exception System.ArgumentNullException: If Strength is set to null
        """
        pass


# noinspection PyPep8Naming
class IConcreteCrackCalculationParameters(metaclass=ABCMeta):
    """The material parameters used in crack calculations to the current design code.
    In some instances these may be different to the material's stress-strain curves.
    """

    @property
    def CharacteristicTensileStrength(self) -> Pressure:
        """Characteristic tensile strength.
        """
        pass

    @property
    def CharacteristicCompressiveStrength(self) -> Pressure:
        """Characteristic compressive strength.
        """
        pass

    @property
    def ElasticModulus(self) -> Pressure:
        """Elastic modulus.
        """
        pass

    @staticmethod
    def Create(elasticModulus: Pressure, characteristicCompressiveStrength: Pressure, characteristicTensileStrength: Pressure) -> IConcreteCrackCalculationParameters:
        """Creates a new "IConcreteCrackCalculationParameters" from
         "Property ElasticModulus",
         "Property CharacteristicCompressiveStrength" and
         "Property CharacteristicTensileStrength".
        Elastic modulus.Characteristic compressive strength.Characteristic tensile strength.

        :exception System.ArgumentException: When the elastic modulus is not a positive and finite value.
        :exception System.ArgumentException: When the characteristic compressive strength is not a negative and finite value.
        :exception System.ArgumentException: When the characteristic tensile strength is not a positive and finite value.
        """
        pass


# noinspection PyPep8Naming
class IConcrete(IMaterial, metaclass=ABCMeta):
    """A concrete material, defined by its stress-strain relationships for both strength and serviceability analysis.

    For pre-defined standard concrete materials, see "Oasys.AdSec.StandardMaterials"
    """

    @property
    def ConcreteCrackCalculationParameters(self) -> IConcreteCrackCalculationParameters:
        """The material parameters used in crack calculations.
        """
        pass

    @staticmethod
    @typing.overload
    def Create(strength: ITensionCompressionCurve, serviceability: ITensionCompressionCurve, concreteCrackCalculationParameters: IConcreteCrackCalculationParameters) -> IConcrete:
        """Creates a new concrete material from its strength (ULS),
        serviceability (SLS) stress-strain relationship and
        material parameters required for crack calculations.

        :param strength: The material's stress-strain relationship for strength (ULS) analysis.
        :type strength: ITensionCompressionCurve
        :param serviceability: The material's stress-strain relationship for serviceability (SLS) analysis.
        :type serviceability: ITensionCompressionCurve
        :param concreteCrackCalculationParameters: The material parameters required for crack calculations.
        :type concreteCrackCalculationParameters: IConcreteCrackCalculationParameters
        :return: A new concrete material with the given sterss-strain relationships.
        :rtype: IConcrete

        :exception System.ArgumentNullException: If strength is null
        :exception System.ArgumentNullException: If serviceability is null
        :exception System.ArgumentNullException: If concreteCrackCalculationParameters is null
        """
        pass

    @staticmethod
    @typing.overload
    def Create(strength: ITensionCompressionCurve, serviceability: ITensionCompressionCurve) -> IConcrete:
        """Creates a new concrete material from its strength (ULS) and
        serviceability (SLS) stress-strain relationship.

        :param strength: The material's stress-strain relationship for strength (ULS) analysis.
        :type strength: ITensionCompressionCurve
        :param serviceability: The material's stress-strain relationship for serviceability (SLS) analysis.
        :type serviceability: ITensionCompressionCurve
        :return: A new concrete material with the given sterss-strain relationships.
        :rtype: IConcrete

        :exception System.ArgumentNullException: If strength is null
        :exception System.ArgumentNullException: If serviceability is null
        """
        pass


# noinspection PyPep8Naming
class IFrp(IMaterial, metaclass=ABCMeta):
    """A fibre-reinforced polymer (FRP) material, defined by its stress-strain relationships for both strength and serviceability analysis.

    For pre-defined standard FRP materials, see "Oasys.AdSec.StandardMaterials"
    """

    @staticmethod
    def Create(strength: ITensionCompressionCurve, serviceability: ITensionCompressionCurve) -> IFrp:
        """Creates a new FRP material from its strength (ULS) and
        serviceability (SLS) stress-strain relationship.

        :param strength: The material's stress-strain relationship for strength (ULS) analysis.
        :type strength: ITensionCompressionCurve
        :param serviceability: The material's stress-strain relationship for serviceability (SLS) analysis.
        :type serviceability: ITensionCompressionCurve
        :return: A new FRP material with the given sterss-strain relationships.
        :rtype: IFrp

        :exception System.ArgumentNullException: If strength is null
        :exception System.ArgumentNullException: If serviceability is null
        """
        pass


# noinspection PyPep8Naming
class IReinforcement(IMaterial, metaclass=ABCMeta):
    """A reinforcement material, defined by its stress-strain relationships for both strength and serviceability analysis.

    For pre-defined standard reinforcement materials, see "Oasys.AdSec.StandardMaterials"
    """

    @staticmethod
    def Create(strength: ITensionCompressionCurve, serviceability: ITensionCompressionCurve) -> IReinforcement:
        """Creates a new reinforcement material from its strength (ULS) and
        serviceability (SLS) stress-strain relationship.

        :param strength: The material's stress-strain relationship for strength (ULS) analysis.
        :type strength: ITensionCompressionCurve
        :param serviceability: The material's stress-strain relationship for serviceability (SLS) analysis.
        :type serviceability: ITensionCompressionCurve
        :return: A new reinforcement material with the given sterss-strain relationships.
        :rtype: IReinforcement

        :exception System.ArgumentNullException: If strength is null
        :exception System.ArgumentNullException: If serviceability is null
        """
        pass


# noinspection PyPep8Naming
class ISteel(IMaterial, metaclass=ABCMeta):
    """A steel material, defined by its stress-strain relationships for both strength and serviceability analysis.

    For pre-defined standard steel materials, see "Oasys.AdSec.StandardMaterials"
    """

    @staticmethod
    def Create(strength: ITensionCompressionCurve, serviceability: ITensionCompressionCurve) -> ISteel:
        """Creates a new steel material from its strength (ULS) and
        serviceability (SLS) stress-strain relationship.

        :param strength: The material's stress-strain relationship for strength (ULS) analysis.
        :type strength: ITensionCompressionCurve
        :param serviceability: The material's stress-strain relationship for serviceability (SLS) analysis.
        :type serviceability: ITensionCompressionCurve
        :return: A new steel material with the given sterss-strain relationships.
        :rtype: ISteel

        :exception System.ArgumentNullException: If strength is null
        :exception System.ArgumentNullException: If serviceability is null
        """
        pass
