#------------------------------------------------------------------------------
# <auto-generated>
#     This file was auto generated.
#
#     Changes to this file may cause incorrect behavior and will be lost if
#     the code is regenerated.
# </auto-generated>
#------------------------------------------------------------------------------
import typing
from abc import abstractmethod, ABCMeta
from Oasys.AdSec.Reinforcement import IBarBundle
from OasysUnits import Length


# noinspection PyPep8Naming
class ILayer(metaclass=ABCMeta):
    """A reinforcement layer.
    Layers define the number of bars in each reinforcement bundle, their diameter and material.

    See Glossary
    Reinforcement layers are used by reinforcement groups,
     "Oasys.AdSec.Reinforcement.Groups.IGroup".

    ILayer is a base class.

    Use "ILayerByBarCount" to define layers by number of bundles or single bars.

    Use "ILayerByBarPitch" to define layers by the "ILayerByBarPitch.Pitch" between bundles or single bars.
    """

    @property
    def BarBundle(self) -> IBarBundle:
        """The bar (or bundle of bars) that will be placed at each position.

        :exception System.ArgumentNullException: If BarBundle is set to null
        """
        pass

    @BarBundle.setter
    def BarBundle(self, value: IBarBundle) -> None: ...


# noinspection PyPep8Naming
class ILayerByBarCount(ILayer, metaclass=ABCMeta):
    """A reinforcement layer defined by the number of bundles or single bars in the layer.
     "ILayerByBarCount" inherits "ILayer".

    It defines a reinforcement layer by the number of bundles or single bars.The bundles or single bars are spaced out evenly over the available space.
    """

    @property
    def Count(self) -> int:
        """The number of bars in the layer.

        :exception System.ArgumentException: When the number of bars is not a positive and finite value.
        """
        pass

    @Count.setter
    def Count(self, value: int) -> None: ...

    @staticmethod
    def Create(count: int, barBundle: IBarBundle) -> ILayerByBarCount:
        """Creates a new "ILayerByBarCount" from the position
        count and bar (or bundle of bars) to be placed at each position.

        :param count: Defines the number of bars in the layer.
        :type count: int
        :param barBundle: Defines the bar (or bundle of bars) that will be placed at each position.
        :type barBundle: IBarBundle
        :return: Reinforcement layer with the specified bar count and bar bundle.
        :rtype: ILayerByBarCount

        :exception System.ArgumentException: When the count is not a positive and finite value.
        :exception System.ArgumentNullException: If barBundle is null
        """
        pass


# noinspection PyPep8Naming
class ILayerByBarPitch(ILayer, metaclass=ABCMeta):
    """A reinforcement layer defined by the bar pitch.
    The number of bars is calculated based on the available length and the given bar pitch.
    The bar pitch is re-calculated to place the bars at equal spacing, with a maximum final pitch of the given value.
    Example: If the available length for the bars is 1000mm and the given bar pitch is 300mm,
    then the number of spacings that can fit in the available length is calculated as 1000 / 300 i.e. 3.333.
    The number of spacings is rounded up (3.333 rounds up to 4) and the bar pitch re-calculated (1000mm / 4), resulting in a final pitch of 250mm.
     "ILayerByBarPitch" implements "ILayer".

    It defines a reinforcement layer by the "Property Pitch".
    """

    @property
    def Pitch(self) -> Length:
        """The pitch is the maximum centre-to-centre spacing of the bundles or single bars.
        The pitch defines the centre-to-centre spacing between bundles or single bars in the layer.

        The number of bars is calculated based on the available length and the given bar pitch.The bar pitch is re-calculated to place the bars at equal spacing, with a maximum final pitch of the given value.

        :exception System.ArgumentException: When the pitch/spacing of bars is not a positive and finite value.
        """
        pass

    @Pitch.setter
    def Pitch(self, value: Length) -> None: ...

    @staticmethod
    def Create(barBundle: IBarBundle, pitch: Length) -> ILayerByBarPitch:
        """Creates a new "ILayerByBarPitch" from the centre-to-centre
        spacing (pitch) and bar (or bundle of bars) to be placed.
        The pitch defines the centre-to-centre spacing between bundles or single bars in the layer.
        The number of bars is calculated based on the available length and the given bar pitch.
        The bar pitch is re-calculated to place the bars at equal spacing, with a maximum final pitch of the given value.

        :param barBundle: Defines the bar (or bundle of bars) that will be placed at each position.
        :type barBundle: IBarBundle
        :param pitch: Defines the centre-to-centre spacing of bar bundles in the layer.
        :type pitch: Length
        :return: Reinforcement layer with the specified bar bundle and pitch.
        :rtype: ILayerByBarPitch

        :exception System.ArgumentException: When the pitch is not a positive and finite value.
        :exception System.ArgumentNullException: If barBundle is null
        """
        pass
